-- I need isobands

love.window.setTitle( 'isolines-and-isobands-02' )


local tileSize = 200

local function drawGrid()
	love.graphics.setColor(0, 0.5,0)
	local w = math.ceil( love.graphics.getWidth() / tileSize )
	local h = math.ceil( love.graphics.getHeight() / tileSize )
	local width, height = love.graphics.getDimensions()	
	for r=0, w do love.graphics.line(r * tileSize, 0, r * tileSize, height) end	
	for c=0, h do love.graphics.line(0, c * tileSize, width, c * tileSize) end
end

function transponieren (udata)
	local tdata = {}
	for y, xs in pairs (udata) do
		for x, value in pairs (xs) do
			tdata[x]=tdata[x]or{}
			tdata[x][y]=value
		end
	end
	return tdata
end


function get_data ()
	local udata = { -- pollution in chunks, 3x3
		{  1,   0, 0.7},
		{0.9, 0.5,   1},
		{  0, 0.1,   1},
	}
	udata = transponieren (udata)
	local data = {}
	for i=1, 2 do
		data[i]={}
		for j=1, 2 do
			data[i][j]=(udata[i][j]+udata[i+1][j]+udata[i][j+1]+udata[i+1][j+1])/4
		end
	end
	data.middle=data[2][2]
	return data, udata
end

function get_levels (data)
--	local levels = {0.2, 0.4, 0.6, 0.8}
--	local levels = {0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9}
	local t = 0.025
	local min = 0+t
	local max = 1-t
	local levels = {}
	for i = min, max, t do
		levels[#levels+1]=i
	end
	
	return levels
end

function genTextes (data)
	local textes = {}
	for i = 1, #data do
		for j = 1, #data[1] do
			local text = {}
			text.x=i*tileSize
			text.y=j*tileSize
			text.text=data[i][j]
			textes[#textes+1]=text
		end
	end
	return textes
end

function drawTextes(textes)
	love.graphics.setColor(0,1,0)
	for i, text in pairs (textes) do
		love.graphics.print(text.text,text.x-20,text.y,0)
	end
end

function as (a, b)
	if not a or not b then return false end
	if a > 1 and b>1 then 
		a=math.floor(a + 0.5)
		b=math.floor(b + 0.5)
	end
	return (math.floor (a*1000) == math.floor (b*1000)) and true or false
end

function in_range ( value, min, max)
	min=min or tileSize
	max=max or 2*tileSize
	return (min<=value) and (value<=max) and true or false
end



function genMPoints(data, levels)
	local mpoints = {}
--	local a,b,c,d,e  = data[1][1], data[1][2], data[2][2], data[2][1], data.middle
	local a,b,c,d,e  = data[1][1], data[2][1], data[2][2], data[1][2], data.middle
	for k, level in pairs (levels) do
		mpoints[level]={}
		local m = mpoints[level]
		
		local xa = tileSize + tileSize*(level-a)/(b-a)
		local ya = tileSize
		if in_range (xa) and in_range (ya) then
			table.insert (mpoints[level], xa)
			table.insert (mpoints[level], ya)
		end
		
		local xb = 2*tileSize
		local yb = tileSize + tileSize*(level-b)/(c-b)
		if not (as(m[#m-1],xb) and as(m[#m],yb)) then
			
			if in_range (xb) and in_range (yb) then
				table.insert (m, xb)
				table.insert (m, yb)
			end
		elseif #m>2 then
			print ('b double: '..level)
			table.insert (m, 1, xb)
			table.insert (m, 2, yb)
		end
		
		local xc = 2*tileSize - tileSize*(level-c)/(d-c)
		local yc = 2*tileSize
		if not (as(m[#m-1],xc) and as(m[#m],yc)) then
			if in_range (xc) and in_range (yc) then
				table.insert (m, xc)
				table.insert (m, yc)
			end
		elseif #m>2 then
			print ('c double: '..level)
			table.insert (m, 1, xc)
			table.insert (m, 2, yc)
		end
		
		local xd = tileSize
		local yd = 2*tileSize - tileSize*(level-d)/(a-d)
--		print ('yd:'..yd)
		if not (as(m[#m-1],xd) and as(m[#m],yd)) then
			if in_range (xd) and in_range (yd) then
				table.insert (m, xd)
				table.insert (m, yd)
			end
		elseif #m>2 then
			print ('d double: '..level)
			table.insert (m, 1, xd)
			table.insert (m, 2, yd)
		end
		
		if #m>2 then
			print (level..': ' .. #m)
			print (unpack(m))
		end
	end
	return mpoints
end

function drawMPoints(mpoints)
	love.graphics.setColor(1,1,1)
	for name, points in pairs (mpoints) do
		love.graphics.points(points)
		for j = 1, #points-1, 2 do
			local x = points[j]
			local y = points[j+1]
			love.graphics.print(name, x-3,y-9, -math.pi/4)
		end
	end
end

function drawLines(mpoints)
	love.graphics.setColor(0,1,1)
	for name, points in pairs (mpoints) do
		local line = {}
		for j = 1, #points-1, 2 do
			local x = points[j]
			local y = points[j+1]
--			if (x >= tileSize and x <= 2*tileSize) 
--				and (y >= tileSize and y <= 2*tileSize) then
				line[#line+1]=x
				line[#line+1]=y
--			end
		end
		if #line > 4 then
			line[#line+1]=line[1]
			line[#line+1]=line[2]
		end
		if #line > 2 then
			love.graphics.line(line)
		end
	end
end

function drawUdata(udata)
	love.graphics.setColor(1,1,0)
	for i = 1, #udata do
		for j = 1, #udata[1] do
			local x=(i-0.5)*tileSize
			local y=(j-0.5)*tileSize
			local text=udata[i][j]
			love.graphics.print(text,x,y-10,0)
		end
	end
end

function love.load()
	love.window.setMode( 600, 600)
	local data, udata = get_data ()
	local levels = get_levels (data)
	local textes = genTextes (data)
	
	local mpoints = genMPoints (data, levels)
	
	
	canvas = love.graphics.newCanvas()
	love.graphics.setCanvas(canvas)
		drawGrid()
		drawUdata(udata)
		drawTextes(textes)
		drawMPoints(mpoints)
		drawLines(mpoints)
	
	love.graphics.setCanvas()
	
--	local data = canvas:newImageData( )
--	data:encode('png')
	
	love.graphics.captureScreenshot( 'file.png' )

end

function love.draw()
	love.graphics.setColor(1,1,1)
	love.graphics.draw(canvas)
end






















